#include <avr/io.h>
#include <util/delay.h>
#include <stdio.h>
#include <string.h>
#include <time.h>
#include <stdlib.h>
#include "spi.h"
#include "ST7735R_TFT.h"
#include "music.h" 

#define BLACK_BTN	PD1
#define WHITE_BTN	PD4
#define LEFT_BTN	PD5
#define RIGHT_BTN	PD6
#define R1		PA0
#define G1		PA1
#define B1		PA2
#define R2		PA3
#define G2		PA4
#define B2		PA5
#define R3		PA6
#define G3		PC6
#define B3		PC5
#define R4		PC4
#define G4		PC3
#define B4		PC2
#define R5		PC1
#define G5		PC0
#define B5		PD7		
#define BUZZ		PD0
#define LEN		21

typedef struct Character
{
	char name[LEN];
	char surname[LEN];
	char shuffled_name[LEN];
	char shuffled_surname[LEN];
	char hint_line1[LEN];
	char hint_line2[LEN];
} Character;

static int frequency[NUM_NOTES] = {262, 294, 330, 349, 392, 440, 494, 523, 587, 659, 698, 784, 880, 988, 679, 416, 467, 623};

void leds_on(int note)
{
	if (note == RE1)
	{
		//G G G G G 
		PORTA |= (1 << G1) | (1 << G2);
		PORTC |= (1 << G3) | (1 << G4) | (1 << G5);
	}
	else if (note == SOL)
	{
		//B G G G B
		PORTA |= (1 << B1) | (1 << G2);
		PORTC |= (1 << G3) | (1 << G4);
		PORTD |= (1 << B5);
	}
	else if (note == SIb)
	{
		//B B G B B 	
		PORTA |= (1 << B1) | (1 << B2);
		PORTC |= (1 << G3) | (1 << B4);
		PORTD |= (1 << B5);
	}
	else if (note == DO1)
	{
		//B B B B B
		PORTA |= (1 << B1) | (1 << B2);
		PORTC |= (1 << B3) | (1 << B4);
		PORTD |= (1 << B5);
	}
	else if (note == MI1b)
	{
		//R B R R R
		PORTA |= (1 << R1) | (1 << B2) | (1 << R3);
		PORTC |= (1 << R4) | (1 << R5);
	}	
	else if (note == FA1)
	{
		//R R B R R
		PORTA |= (1 << R1) | (1 << R2);
		PORTC |= (1 << B3) | (1 << R4) | (1 << R5);
	}	
	else if (note == SOL1 || note == LAb)
	{
		//R R R B R
		PORTA |= (1 << R1) | (1 << R2) | (1 << R3);
		PORTC |= (1 << B4) | (1 << R5);
	}
	else if (note == FA1b)
	{
		//B G G G B
		PORTA |= (1 << B1) | (1 << G2);
		PORTC |= (1 << G3) | (1 << G4);
		PORTD |= (1 << B5);
	}
}

void leds_off(int note)
{
	if (note == RE1)
	{
		//G G G G G 
		PORTA &= ~((1 << G1) | (1 << G2));
		PORTC &= ~((1 << G3) | (1 << G4) | (1 << G5));
	}
	else if (note == SOL)
	{
		//B G G G B
		PORTA &= ~((1 << B1) | (1 << G2));
		PORTC &= ~((1 << G3) | (1 << G4));
		PORTD &= ~(1 << B5);
	}
	else if (note == SIb)
	{
		//B B G B B 	
		PORTA &= ~((1 << B1) | (1 << B2));
		PORTC &= ~((1 << G3) | (1 << B4));
		PORTD &= ~(1 << B5);
	}
	else if (note == DO1)
	{
		//B B B B B
		PORTA &= ~((1 << B1) | (1 << B2));
		PORTC &= ~((1 << B3) | (1 << B4));
		PORTD &= ~(1 << B5);
	}
	else if (note == MI1b)
	{
		//R B R R R
		PORTA &= ~((1 << R1) | (1 << B2) | (1 << R3));
		PORTC &= ~((1 << R4) | (1 << R5));
	}	
	else if (note == FA1)
	{
		//R R B R R
		PORTA &= ~((1 << R1) | (1 << R2));
		PORTC &= ~((1 << B3) | (1 << R4) | (1 << R5));
	}	
	else if (note == SOL1 || note == LAb)
	{
		//R R R B R
		PORTA &= ~((1 << R1) | (1 << R2) | (1 << R3));
		PORTC &= ~((1 << B4) | (1 << R5));
	}
	else if (note == FA1b)
	{
		//B G G G B
		PORTA &= ~((1 << B1) | (1 << G2));
		PORTC &= ~((1 << G3) | (1 << G4));
		PORTD &= ~(1 << B5);
	}
}

//generate a note
static void buzzer_note(int note, int time) {
	int note_period = 1000000 / frequency[note];
	int i, j;
	leds_on(note);
	for (j = 0; j < time; j++) {
		PORTD |= (1 << BUZZ);	
		for (i = 0; i < note_period / 2; ++i)
			_delay_us(1);

		PORTD &= ~(1 << BUZZ);
		for (i = 0; i < note_period / 2; ++i)
			_delay_us(1);
	}
	leds_off(note);
}

void play_song()
{
	while (1) {
		buzzer_note(SOL1, THREE);
		_delay_ms(PAUSE);
		buzzer_note(DO1, THREE);
		_delay_ms(PAUSE);
		buzzer_note(MI1b, HALF);
		_delay_ms(PAUSE);
		buzzer_note(FA1, HALF);
		_delay_ms(PAUSE);
		buzzer_note(SOL1, TWO);
		_delay_ms(PAUSE);
		buzzer_note(DO1, TWO);
		_delay_ms(PAUSE);
		buzzer_note(MI1b, HALF);
		_delay_ms(PAUSE);
		buzzer_note(FA1, HALF);
		_delay_ms(PAUSE);
		buzzer_note(RE1, ONE);
		_delay_ms(PAUSE);
		buzzer_note(SOL, HALF);
		_delay_ms(PAUSE);
		buzzer_note(SIb, HALF);
		_delay_ms(PAUSE);
		buzzer_note(DO1, HALF);
		_delay_ms(PAUSE);
		buzzer_note(RE1, ONE);
		_delay_ms(PAUSE);
		buzzer_note(SOL, HALF);
		_delay_ms(PAUSE);
		buzzer_note(SIb, HALF);
		_delay_ms(PAUSE);
		buzzer_note(DO1, HALF);
		_delay_ms(PAUSE);
		buzzer_note(RE1, ONE);
		_delay_ms(PAUSE);
		buzzer_note(SOL, HALF);
		_delay_ms(PAUSE);
		buzzer_note(SIb, HALF);
		_delay_ms(PAUSE);
		buzzer_note(DO1, HALF);
		_delay_ms(PAUSE);
		buzzer_note(RE1, HALF);
		_delay_ms(PAUSE);
		buzzer_note(SOL, HALF);
		_delay_ms(PAUSE);
		buzzer_note(SIb, HALF);
		_delay_ms(PAUSE);
		buzzer_note(FA1b, THREE);
		_delay_ms(PAUSE);
		buzzer_note(SIb, THREE);
		_delay_ms(PAUSE);
		buzzer_note(MI1b, HALF);
		_delay_ms(PAUSE);
		buzzer_note(RE1, HALF);
		_delay_ms(PAUSE);
		buzzer_note(FA1, TWO);
		_delay_ms(PAUSE);
		buzzer_note(SIb, THREE);
		_delay_ms(PAUSE);
		buzzer_note(MI1b, HALF);
		_delay_ms(PAUSE);
		buzzer_note(RE1, HALF);
		_delay_ms(PAUSE);
		buzzer_note(DO1, ONE);
		_delay_ms(PAUSE);
		buzzer_note(LAb, HALF);
		_delay_ms(PAUSE);
		buzzer_note(MI1b, HALF);
		_delay_ms(PAUSE);
		buzzer_note(RE1, HALF);
		_delay_ms(PAUSE);
		buzzer_note(DO1, ONE);
		_delay_ms(PAUSE);
		buzzer_note(LAb, HALF);
		_delay_ms(PAUSE);
		buzzer_note(MI1b, HALF);
		_delay_ms(PAUSE);
		buzzer_note(RE1, HALF);
		_delay_ms(PAUSE);
		buzzer_note(DO1, ONE);
		_delay_ms(PAUSE);
		buzzer_note(LAb, HALF);
		_delay_ms(PAUSE);
		buzzer_note(MI1b, HALF);
		_delay_ms(PAUSE);
		buzzer_note(RE1, HALF);
		_delay_ms(PAUSE);
		buzzer_note(DO1, ONE);
		_delay_ms(PAUSE);
		buzzer_note(LAb, HALF);
		_delay_ms(10*PAUSE);
	}
}

void print_menu()
{
	ST7735R_FillRect(0, 0, ST7735R_WIDTH - 1, ST7735R_HEIGHT - 1, 0, 0, 0);
	//SWORD
	//sharp point
	ST7735R_Line(10, 24, 20, 20, 211, 211, 211);
	ST7735R_Line(10, 24, 20, 28, 211, 211, 211);
	//long blade
	ST7735R_Line(20, 20, 85, 20, 211, 211, 211);
	ST7735R_Line(10, 24, 85, 24, 211, 211, 211);		
	ST7735R_Line(20, 28, 85, 28, 211, 211, 211);
	//handle bottom
	ST7735R_FillRect(85, 15, 91, 33, 169, 169, 169);
	//handle
	ST7735R_Line(91, 21, 111, 21, 169, 169, 169);
	ST7735R_Line(91, 27, 111, 27, 169, 169, 169);
	ST7735R_FillRect(96, 21, 101, 27, 169, 169, 169);
	ST7735R_FillRect(106, 21, 111, 27, 169, 169, 169);

	//TITLE
	ST7735R_DrawText(46, 45, "A GAME\0", 255, 255, 255, 0, 0, 0);
	ST7735R_DrawText(62, 60, "OF\0", 255, 255, 255, 0, 0, 0);
	ST7735R_DrawText(45, 75, "THRONES\0", 255, 255, 255, 0, 0, 0);
	
	//MENU
	ST7735R_Line(25, 108, 103, 108, 255, 255, 255);
	ST7735R_DrawText(49, 115, "START\0", 255, 255, 255, 0, 0, 0);
	ST7735R_Line(25, 126, 103, 126, 255, 255, 255);
	ST7735R_DrawText(31, 137, "INSTRUCTIONS\0", 255, 255, 255, 0, 0, 0);
	ST7735R_Line(25, 149, 103, 149, 255, 255, 255);
}

void print_instructions()
{
	ST7735R_FillRect(0, 0, ST7735R_WIDTH - 1, ST7735R_HEIGHT - 1, 0, 0, 0);
	ST7735R_DrawText(3, 2, "When you play the \0", 255, 255, 255, 0, 0, 0);
	ST7735R_DrawText(3, 12, "game of thrones, you\0", 255, 255, 255, 0, 0, 0);
	ST7735R_DrawText(3, 22, "win or you die. \0", 255, 255, 255, 0, 0, 0);
	ST7735R_DrawText(3, 32, "Find the right \0", 255, 255, 255, 0, 0, 0);
	ST7735R_DrawText(3, 42, "character and enjoy\0", 255, 255, 255, 0, 0, 0);
	ST7735R_DrawText(3, 52, "the show!\0", 255, 255, 255, 0, 0, 0);
	
	ST7735R_DrawText(13, 81, "BLACK -> SELECT\0", 192, 192, 192, 0, 0, 0);
	ST7735R_DrawText(13, 93, "WHITE -> HINT\0", 255, 255, 255, 0, 0, 0);
	ST7735R_DrawText(13, 105, "RED 1 -> LEFT\0", 255, 0, 0, 0, 0, 0);
	ST7735R_DrawText(13, 117, "RED 2 -> RIGHT\0", 255, 0, 0, 0, 0, 0);
	ST7735R_Line(0, 142, ST7735R_WIDTH - 1, 142, 255, 255, 255);
	ST7735R_DrawText(2, 149, "PRESS BLACK TO START\0", 255, 255, 255, 0, 0, 0);
}

void print_dragon()
{
	ST7735R_FillRect(0, 0, ST7735R_WIDTH - 1, ST7735R_HEIGHT - 1, 0, 0, 0);
	//LETTER BOX
	ST7735R_Line(57, 41, 71, 41, 255, 255, 255);
	ST7735R_Line(71, 41, 71, 55, 255, 255, 255);
	ST7735R_Line(57, 55, 71, 55, 255, 255, 255);
	ST7735R_Line(57, 41, 57, 55, 255, 255, 255);
	//TEXT BOX
	//up
	ST7735R_Line(2, 97, 44, 97, 255, 255, 255);
	ST7735R_Line(56, 97, 59, 97, 255, 255, 255);
	ST7735R_Line(69, 97, 72, 97, 255, 255, 255);
	ST7735R_Line(84, 97, 126, 97, 255, 255, 255);
	//left
	ST7735R_Line(2, 97, 2, 137, 255, 255, 255);
	//right
	ST7735R_Line(126, 97, 126, 137, 255, 255, 255);
	//down
	ST7735R_Line(2, 137, 126, 137, 255, 255, 255);
	//text
	ST7735R_DrawText(7, 110, "In need of whispers\0", 255, 255, 255, 0, 0, 0);
	ST7735R_DrawText(7, 123, "    from a raven?  \0", 255, 255, 255, 0, 0, 0);

	//DRAGON
	//bottom left wings
	ST7735R_Line(10, 138, 4, 157, 60, 179, 113);
	ST7735R_Line(27, 138, 4, 157, 60, 179, 113);
	ST7735R_Line(27, 138, 27, 147, 60, 179, 113);
	ST7735R_Line(43, 138, 27, 147, 60, 179, 113);
	//bottom right wings
	ST7735R_Line(85, 138, 101, 147, 60, 179, 113);
	ST7735R_Line(101, 138, 101, 147, 60, 179, 113);
	ST7735R_Line(101, 138, 124, 157, 60, 179, 113);
	ST7735R_Line(118, 138, 124, 157, 60, 179, 113);
	//tail
	ST7735R_Line( 54, 139, 49, 142, 60, 179, 113);
	ST7735R_Line( 49, 142, 54, 147, 60, 179, 113);
	ST7735R_Line( 54, 147, 49, 152, 60, 179, 113);
	ST7735R_Line( 49, 152, 59, 159, 60, 179, 113);
	ST7735R_Line( 64, 139, 59, 142, 60, 179, 113);
	ST7735R_Line( 59, 142, 64, 147, 60, 179, 113);
	ST7735R_Line( 64, 147, 59, 152, 60, 179, 113);
	ST7735R_Line( 59, 152, 59, 159, 60, 179, 113);
	//head
	ST7735R_Line(64, 81, 64, 105, 60, 179, 113);
	ST7735R_Line(59, 81, 69, 81, 60, 179, 113);
	//left half
	ST7735R_Line(55, 75, 59, 81, 60, 179, 113);
	ST7735R_Line(55, 75, 55, 85, 60, 179, 113);
	ST7735R_Line(59, 81, 53, 89, 60, 179, 113);
	ST7735R_Line(53, 89, 64, 105, 60, 179, 113);
	//right half	
	ST7735R_Line(73, 75, 69, 81, 60, 179, 113);
	ST7735R_Line(73, 75, 73, 85, 60, 179, 113);
	ST7735R_Line(69, 81, 75, 89, 60, 179, 113);
	ST7735R_Line(75, 89, 64, 105, 60, 179, 113);
	//eyes
	ST7735R_FilledCircle(60, 89, 1, 220, 20, 60);
	ST7735R_FilledCircle(68, 89, 1, 220, 20, 60);
	//top left wings
	ST7735R_Line(20, 75, 10, 96, 60, 179, 113);
	ST7735R_Line(20, 75, 20, 96, 60, 179, 113);
	ST7735R_Line(20, 75, 40, 96, 60, 179, 113);
	//top right wings
	ST7735R_Line(108, 75, 118, 96, 60, 179, 113);
	ST7735R_Line(108, 75, 108, 96, 60, 179, 113);
	ST7735R_Line(108, 75, 88, 96, 60, 179, 113);
	//left claws
	ST7735R_Circle( 46, 97, 2, 60, 179, 113);
	ST7735R_Circle( 50, 97, 2, 60, 179, 113);
	ST7735R_Circle( 54, 97, 2, 60, 179, 113);
	ST7735R_Line(46, 99, 46, 101, 255, 255, 255);
	ST7735R_Line(50, 99, 50, 101, 255, 255, 255);
	ST7735R_Line(54, 99, 54, 101, 255, 255, 255);
	//right claws
	ST7735R_Circle( 74, 97, 2, 60, 179, 113);	
	ST7735R_Circle( 78, 97, 2, 60, 179, 113);
	ST7735R_Circle( 82, 97, 2, 60, 179, 113);
	ST7735R_Line(74, 99, 74, 101, 255, 255, 255);
	ST7735R_Line(78, 99, 78, 101, 255, 255, 255);
	ST7735R_Line(82, 99, 82, 101, 255, 255, 255);
}

void print_hint(char line1[LEN], char line2[LEN])
{
	ST7735R_FillRect(5, 102, 123, 128, 0, 0, 0);
	ST7735R_DrawText(5, 108, line1, 255, 255, 255, 0, 0, 0);
	ST7735R_DrawText(5, 121, line2, 255, 255, 255, 0, 0, 0);
}

void print_words(char name[LEN], char surname[LEN], int len1, int len2, int cursor, int center_1, int center_2)
{
	int pos = 0;
	for (int i = 0; i < len1; i++)
	{
		pos = center_1 + 3 + i * 12;
		char aux[2];
		aux[0] = name[i];
		aux[1] = '\0';
		ST7735R_DrawText(pos, 7, aux, 255, 255, 255, 0, 0, 0);
		//cursor on first vector
		if (cursor == i) 
			ST7735R_Line(pos, 17, pos + 5, 17, 255, 255, 255);
	}
	for (int i = 0; i < len2; i++)
	{
		pos = center_2 + 3 + i * 12;
		char aux[2];
		aux[0] = surname[i];
		aux[1] = '\0';
		ST7735R_DrawText(pos, 22, aux, 255, 255, 255, 0, 0, 0);
	}	
		
}

void update_words(char name[LEN], char surname[LEN], int len1, int len2, int cursor, int old_cursor, int center_1, int center_2, char temp[2])
{
	if (cursor != old_cursor)
	{
		//check on which vector is the cursor
		if (cursor < len1)
			ST7735R_Line(3 + center_1 + cursor * 12, 17, 8 + center_1 + cursor * 12, 17, 255, 255, 255);
		else
			ST7735R_Line(3 + center_2 + (cursor - len1) * 12, 32, 8 + center_2 + (cursor - len1) * 12, 32, 255, 255, 255);
		//check on which vector was the old cursor
		if (old_cursor < len1)
			ST7735R_FillRect(2 + center_1 + old_cursor * 12, 16, 9 + center_1 + old_cursor * 12, 18, 0, 0, 0);
		else
			ST7735R_FillRect(2 + center_2 + (old_cursor - len1) * 12, 31, 9 + center_2 + (old_cursor - len1) * 12, 33, 0, 0, 0);
	}

	if (temp[0] != temp[1])
	{
		//erase letter pointed by cursor and rewrite it
		char aux[2];
		aux[0] = temp[1];
		aux[1] = '\0';
		if (cursor < len1)
		{
			ST7735R_FillRect(2 + center_1 + cursor * 12, 6, 7 + center_1 + cursor * 12, 16, 0, 0, 0);
			ST7735R_DrawText(3 + center_1 + cursor * 12, 7, aux, 255, 255, 255, 0, 0, 0);
		}
		else if (cursor >= len1)
		{
			ST7735R_FillRect(2 + center_2 + (cursor - len1) * 12, 21, 7 + center_2 + (cursor - len1) * 12, 31, 0, 0, 0);
			ST7735R_DrawText(3 + center_2 + (cursor - len1) * 12, 22, aux, 255, 255, 255, 0, 0, 0);
		}	
		//erase letter box and rewrite it
		aux[0] = temp[0];
		aux[1] = '\0';
		ST7735R_FillRect(61, 44, 68, 54, 0, 0, 0);
		ST7735R_DrawText(62, 45, aux, 255, 255, 255, 0, 0, 0);
		temp[1] = temp[0];
	}	
}

void shuffle(char word[LEN])
{
	int i, j, len;
	char temp;

	for (i = 0; word[i] != '\0'; )
	    i++;
	len = i;

	srand(time(NULL));

	for (i = 0; i < len; i++)
	{
		j = rand() % len;
		temp = word[i];
		word[i] = word[j];
		word[j] = temp;
	}
}

void print_won_panel()
{
	ST7735R_FillRect(0, 0, ST7735R_WIDTH - 1, ST7735R_HEIGHT - 1, 0, 0, 0);
	ST7735R_DrawText(40, 31, "You won!\0", 255, 255, 255, 0, 0, 0);
	//crown
	ST7735R_Line(32, 54, 48, 68, 255, 215, 0);
	ST7735R_Line(48, 68, 64, 54, 255, 215, 0);
	ST7735R_Line(64, 54, 80, 68, 255, 215, 0);
	ST7735R_Line(80, 68, 96, 54, 255, 215, 0);
	ST7735R_Line(96, 54, 96, 81, 255, 215, 0);
	ST7735R_Line(32, 81, 96, 81, 255, 215, 0);
	ST7735R_Line(32, 54, 32, 81, 255, 215, 0);
	//more text
	ST7735R_DrawText(3, 100, "Just the game,not the\0", 255, 255, 255, 0, 0, 0);
	ST7735R_DrawText(3, 110, "crown.That's just for\0", 255, 255, 255, 0, 0, 0);
	ST7735R_DrawText(3, 120, "display. \0", 255, 255, 255, 0, 0, 0);
	
	ST7735R_Line(0, 142, ST7735R_WIDTH - 1, 142, 255, 255, 255);
	ST7735R_DrawText(2, 149, "PRESS RESET TO REPLAY\0", 255, 255, 255, 0, 0, 0);
}

int main()
{
	// Init SPI for LCD. Init LCD.
    	SPI_init();
    	ST7735R_Begin();

	Character people[4];
	strcpy(people[0].name, "NED\0");
	strcpy(people[0].surname, "STARK\0");
	strcpy(people[0].shuffled_name, "NED\0");
	strcpy(people[0].shuffled_surname, "STARK\0");
	strcpy(people[0].hint_line1, "   Beheaded on the  \0");
	strcpy(people[0].hint_line2, "  orders of a prick \0");

	strcpy(people[1].name, "JAMIE\0");
	strcpy(people[1].surname, "LANNISTER\0");
	strcpy(people[1].shuffled_name, "JAMIE\0");
	strcpy(people[1].shuffled_surname, "LANNISTER\0");
	strcpy(people[1].hint_line1, "   Killed by rocks, \0");
	strcpy(people[1].hint_line2, " holding his sister \0");

	strcpy(people[2].name, "JON\0");
	strcpy(people[2].surname, "SNOW\0");
	strcpy(people[2].shuffled_name, "JON\0");
	strcpy(people[2].shuffled_surname, "SNOW\0");
	strcpy(people[2].hint_line1, " Stabbed by traitors\0");
	strcpy(people[2].hint_line2, "(but wakes up later)\0");

	strcpy(people[3].name, "OBERYN\0");
	strcpy(people[3].surname, "MARTELL\0");
	strcpy(people[3].shuffled_name, "OBERYN\0");
	strcpy(people[3].shuffled_surname, "MARTELL\0");
	strcpy(people[3].hint_line1, " Crushed by Clegane \0");
	strcpy(people[3].hint_line2, " in Trial by Combat \0");

	// Set input pins (buttons)
	DDRD &= ~((1 << PD1) | (1 << PD4) | (1 << PD5) | (1 << PD6));
	// Activate pull-up
	PORTD |= (1 << PD1) | (1 << PD4) | (1 << PD5) | (1 << PD6);

	//Set output pins (LEDs and buzzer)
	DDRA |= (1 << PA0) | (1 << PA1) | (1 << PA2) | (1 << PA3) | (1 << PA4) | (1 << PA5) | (1 << PA6);
	DDRC |= (1 << PC0) | (1 << PC1) | (1 << PC2) | (1 << PC3) | (1 << PC4) | (1 << PC5) | (1 << PC6);
	DDRD |= (1 << PD0) | (1 << PD7);
	//Turn off anything that might be on
	PORTA &= ~((1 << PA0) | (1 << PA1) | (1 << PA2) | (1 << PA3) | (1 << PA4) | (1 << PA5) | (1 << PA6));
	PORTC &= ~((1 << PC0) | (1 << PC1) | (1 << PC2) | (1 << PC3) | (1 << PC4) | (1 << PC5) | (1 << PC6));
	PORTD &= ~((1 << PD0) | (1 << PD7));

    	// Clear screen with BLACK.
    	ST7735R_FillRect(0, 0, ST7735R_WIDTH - 1, ST7735R_HEIGHT - 1, 0, 0, 0);
	print_menu();
	while (1)
	{
		if ((PIND & (1 << BLACK_BTN)) == 0)
			break;
		if ((PIND & (1 << WHITE_BTN)) == 0)
		{
			print_instructions();
			while((PIND & (1 << BLACK_BTN)));
			break;
		}
	}
	_delay_ms(500);
	print_dragon();
	//get a random character index from the array
	srand(time(NULL));
	int chosen = rand() % 4;
	//shuffle names
	shuffle(people[chosen].shuffled_name);
	shuffle(people[chosen].shuffled_surname);

	//get names length
	int i;
	for (i = 0; people[chosen].name[i] != '\0'; )
	    i++;
	int len1 = i;
	for (i = 0; people[chosen].surname[i] != '\0'; )
	    i++;
	int len2 = i;

	//set state
	int won = 0;
	//set initial position
	int cursor = 0;
	int old_cursor = 0;
	//get index for centering the text
	int center_1 = (125 - (len1 * 12 + 3)) / 2;
	int center_2 = (125 - (len2 * 12 + 3)) / 2;
	//set auxiliary char
	char temp[2];
	temp[0] = ' ';
	temp[1] = ' ';

	//print the words char by char	
	print_words(people[chosen].shuffled_name, people[chosen].shuffled_surname, len1, len2, cursor, center_1, center_2);
	while(1)
	{
		if (strcmp(people[chosen].shuffled_name, people[chosen].name) == 0 && strcmp(people[chosen].shuffled_surname, people[chosen].surname) == 0)
		{
			won = 1;
			break;
		}
		//show hint
		if ((PIND & (1 << WHITE_BTN)) == 0)
		{
			print_hint(people[chosen].hint_line1, people[chosen].hint_line2);
			_delay_ms(500);
		}		
		//select and switch letters
		if ((PIND & (1 << BLACK_BTN)) == 0)
		{
			if (cursor < len1)
			{
				temp[0] = people[chosen].shuffled_name[cursor];
				people[chosen].shuffled_name[cursor] = temp[1];
			}
			else
			{
				int pos = cursor - len1;
				temp[0] = people[chosen].shuffled_surname[pos];
				people[chosen].shuffled_surname[pos] = temp[1];
			}
			update_words(people[chosen].shuffled_name, people[chosen].shuffled_surname, len1, len2, cursor, old_cursor, center_1, center_2, temp);
			_delay_ms(500);	
		}
		//move right
		if ((PIND & (1 << RIGHT_BTN)) == 0)
		{
			old_cursor = cursor;
			cursor  = (cursor + 1) % (len1 + len2);
			update_words(people[chosen].shuffled_name, people[chosen].shuffled_surname, len1, len2, cursor, old_cursor, center_1, center_2, temp);
			_delay_ms(500);
		}
		//move left
		if ((PIND & (1 << LEFT_BTN)) == 0)
		{
			old_cursor = cursor;
			cursor--;
			if (cursor < 0)
				cursor = len1 + len2 - 1;
			update_words(people[chosen].shuffled_name, people[chosen].shuffled_surname, len1, len2, cursor, old_cursor, center_1, center_2, temp);
			_delay_ms(500);
		}
	}
	if (won)
	{
		print_won_panel();
		play_song();
	}
	return 0;
}










